# DO NOT EDIT THIS FILE!
# This file was automatically generated using the timeeval_experiments.generator from the template:
# timeeval_experiments/generator/templates/docker-algorithm.py.jinja
from durations import Duration
from typing import Any, Dict, Optional

from timeeval import Algorithm, TrainingType, InputDimensionality
from timeeval.adapters import DockerAdapter
from timeeval.params import ParameterConfig


_autoencoder_parameters: Dict[str, Dict[str, Any]] = {
 "early_stopping_delta": {
  "defaultValue": 0.5,
  "description": "If loss is `delta` or less smaller for `patience` epochs, stop",
  "name": "early_stopping_delta",
  "type": "float"
 },
 "early_stopping_patience": {
  "defaultValue": 10,
  "description": "If loss is `delta` or less smaller for `patience` epochs, stop",
  "name": "early_stopping_patience",
  "type": "int"
 },
 "epochs": {
  "defaultValue": 10,
  "description": "Number of training epochs",
  "name": "epochs",
  "type": "int"
 },
 "latent_size": {
  "defaultValue": 32,
  "description": "Dimensionality of the latent space",
  "name": "latent_size",
  "type": "int"
 },
 "learning_rate": {
  "defaultValue": "0.005",
  "description": "Learning rate",
  "name": "learning_rate",
  "type": "float"
 },
 "random_state": {
  "defaultValue": 42,
  "description": "Seed for the random number generator",
  "name": "random_state",
  "type": "int"
 },
 "split": {
  "defaultValue": 0.8,
  "description": "Fraction to split training data by for validation",
  "name": "split",
  "type": "float"
 }
}


def autoencoder(params: Optional[ParameterConfig] = None, skip_pull: bool = False, timeout: Optional[Duration] = None) -> Algorithm:
    """AutoEncoder (AE)

    Implementation of https://dl.acm.org/doi/10.1145/2689746.2689747


    **Algorithm Parameters:**

    latent_size: int
        Dimensionality of the latent space (default: ``32``)
    epochs: int
        Number of training epochs (default: ``10``)
    learning_rate: float
        Learning rate (default: ``0.005``)
    split: float
        Fraction to split training data by for validation (default: ``0.8``)
    early_stopping_delta: float
        If loss is `delta` or less smaller for `patience` epochs, stop (default: ``0.5``)
    early_stopping_patience: int
        If loss is `delta` or less smaller for `patience` epochs, stop (default: ``10``)
    random_state: int
        Seed for the random number generator (default: ``42``)

    Parameters
    ----------
    params : Optional[ParameterConfig]
        Parameter configuration for the algorithm
    skip_pull : bool
        Set to ``True`` to skip pulling the Docker image and use a local image instead.
        If the image is not present locally, this will raise an error.
    timeout : Optional[Duration]
        Set an individual execution and training timeout for this algorithm.
        This will overwrite the global timeouts set using :class:`~timeeval.ResourceConstraints`.

    Returns
    -------
    ~timeeval.Algorithm
        A correctly configured :class:`~timeeval.Algorithm` object for the AutoEncoder (AE) algorithm.
    """
    return Algorithm(
        name="AutoEncoder (AE)",
        main=DockerAdapter(
            image_name="ghcr.io/timeeval/autoencoder",
            tag="0.3.0",
            skip_pull=skip_pull,
            timeout=timeout,
            group_privileges="akita",
        ),
        preprocess=None,
        postprocess=None,
        param_schema=_autoencoder_parameters,
        param_config=params or ParameterConfig.defaults(),
        data_as_file=True,
        training_type=TrainingType.SEMI_SUPERVISED,
        input_dimensionality=InputDimensionality("multivariate")
    )
