# DO NOT EDIT THIS FILE!
# This file was automatically generated using the timeeval_experiments.generator from the template:
# timeeval_experiments/generator/templates/docker-algorithm.py.jinja
from durations import Duration
from typing import Any, Dict, Optional

from timeeval import Algorithm, TrainingType, InputDimensionality
from timeeval.adapters import DockerAdapter
from timeeval.params import ParameterConfig


_ensemble_gi_parameters: Dict[str, Dict[str, Any]] = {
 "anomaly_window_size": {
  "defaultValue": 50,
  "description": "The size of the sliding window, in which `w` regions are made discrete.",
  "name": "anomaly_window_size",
  "type": "int"
 },
 "max_alphabet_size": {
  "defaultValue": 10,
  "description": "Maximum number of symbols used for discretization by SAX (`\\alpha`)",
  "name": "max_alphabet_size",
  "type": "int"
 },
 "max_paa_transform_size": {
  "defaultValue": 20,
  "description": "Maximum size of the embedding space used by PAA (SAX word size `w`)",
  "name": "max_paa_transform_size",
  "type": "int"
 },
 "n_estimators": {
  "defaultValue": 10,
  "description": "The number of models in the ensemble.",
  "name": "n_estimators",
  "type": "int"
 },
 "n_jobs": {
  "defaultValue": 1,
  "description": "The number of parallel jobs to use for executing the models. If `-1`, then the number of jobs is set to the number of CPU cores.",
  "name": "n_jobs",
  "type": "int"
 },
 "random_state": {
  "defaultValue": 42,
  "description": "Seed for the random number generator",
  "name": "random_state",
  "type": "int"
 },
 "selectivity": {
  "defaultValue": 0.8,
  "description": "The fraction of models in the ensemble included in the end result.",
  "name": "selectivity",
  "type": "float"
 },
 "window_method": {
  "defaultValue": "sliding",
  "description": "Windowing method used to create subsequences. The original implementation had a strange method (`orig`) that is similar to `tumbling`, the paper uses a `sliding` window. However, `sliding` is significantly slower than `tumbling` while producing better results (higher anomaly score resolution). `orig` should not be used!",
  "name": "window_method",
  "type": "enum[sliding,tumbling,orig]"
 }
}


def ensemble_gi(params: Optional[ParameterConfig] = None, skip_pull: bool = False, timeout: Optional[Duration] = None) -> Algorithm:
    """Ensemble GI

    Implementation of https://doi.org/10.5441/002/edbt.2020.09


    **Algorithm Parameters:**

    anomaly_window_size: int
        The size of the sliding window, in which `w` regions are made discrete. (default: ``50``)
    n_estimators: int
        The number of models in the ensemble. (default: ``10``)
    max_paa_transform_size: int
        Maximum size of the embedding space used by PAA (SAX word size `w`) (default: ``20``)
    max_alphabet_size: int
        Maximum number of symbols used for discretization by SAX (`\alpha`) (default: ``10``)
    selectivity: float
        The fraction of models in the ensemble included in the end result. (default: ``0.8``)
    random_state: int
        Seed for the random number generator (default: ``42``)
    n_jobs: int
        The number of parallel jobs to use for executing the models. If `-1`, then the number of jobs is set to the number of CPU cores. (default: ``1``)
    window_method: enum[sliding,tumbling,orig]
        Windowing method used to create subsequences. The original implementation had a strange method (`orig`) that is similar to `tumbling`, the paper uses a `sliding` window. However, `sliding` is significantly slower than `tumbling` while producing better results (higher anomaly score resolution). `orig` should not be used! (default: ``sliding``)

    Parameters
    ----------
    params : Optional[ParameterConfig]
        Parameter configuration for the algorithm
    skip_pull : bool
        Set to ``True`` to skip pulling the Docker image and use a local image instead.
        If the image is not present locally, this will raise an error.
    timeout : Optional[Duration]
        Set an individual execution and training timeout for this algorithm.
        This will overwrite the global timeouts set using :class:`~timeeval.ResourceConstraints`.

    Returns
    -------
    ~timeeval.Algorithm
        A correctly configured :class:`~timeeval.Algorithm` object for the Ensemble GI algorithm.
    """
    return Algorithm(
        name="Ensemble GI",
        main=DockerAdapter(
            image_name="ghcr.io/timeeval/ensemble_gi",
            tag="0.3.0",
            skip_pull=skip_pull,
            timeout=timeout,
            group_privileges="akita",
        ),
        preprocess=None,
        postprocess=None,
        param_schema=_ensemble_gi_parameters,
        param_config=params or ParameterConfig.defaults(),
        data_as_file=True,
        training_type=TrainingType.UNSUPERVISED,
        input_dimensionality=InputDimensionality("univariate")
    )
