# DO NOT EDIT THIS FILE!
# This file was automatically generated using the timeeval_experiments.generator from the template:
# timeeval_experiments/generator/templates/docker-algorithm.py.jinja
from durations import Duration
from typing import Any, Dict, Optional

from timeeval import Algorithm, TrainingType, InputDimensionality
from timeeval.adapters import DockerAdapter
from timeeval.params import ParameterConfig


_lstm_vae_parameters: Dict[str, Dict[str, Any]] = {
 "batch_size": {
  "defaultValue": 32,
  "description": "size of batch given for each iteration",
  "name": "batch_size",
  "type": "int"
 },
 "early_stopping_delta": {
  "defaultValue": 0.05,
  "description": "If 1 - (loss / last_loss) is less than `delta` for `patience` epochs, stop",
  "name": "early_stopping_delta",
  "type": "float"
 },
 "early_stopping_patience": {
  "defaultValue": 10,
  "description": "If 1 - (loss / last_loss) is less than `delta` for `patience` epochs, stop",
  "name": "early_stopping_patience",
  "type": "int"
 },
 "epochs": {
  "defaultValue": 10,
  "description": "number of iterations we train the model",
  "name": "epochs",
  "type": "int"
 },
 "latent_size": {
  "defaultValue": 5,
  "description": "dimension of latent space",
  "name": "latent_size",
  "type": "int"
 },
 "learning_rate": {
  "defaultValue": 0.001,
  "description": "rate at which the gradients are updated",
  "name": "learning_rate",
  "type": "float"
 },
 "lstm_layers": {
  "defaultValue": 10,
  "description": "number of layers in lstm",
  "name": "lstm_layers",
  "type": "int"
 },
 "rnn_hidden_size": {
  "defaultValue": 5,
  "description": "LTSM cells hidden dimension",
  "name": "rnn_hidden_size",
  "type": "int"
 },
 "window_size": {
  "defaultValue": 10,
  "description": "number of datapoints that the model takes once",
  "name": "window_size",
  "type": "int"
 }
}


def lstm_vae(params: Optional[ParameterConfig] = None, skip_pull: bool = False, timeout: Optional[Duration] = None) -> Algorithm:
    """LSTM-VAE

    self implementation of: https://ieeexplore.ieee.org/document/8279425


    **Algorithm Parameters:**

    rnn_hidden_size: int
        LTSM cells hidden dimension (default: ``5``)
    latent_size: int
        dimension of latent space (default: ``5``)
    learning_rate: float
        rate at which the gradients are updated (default: ``0.001``)
    batch_size: int
        size of batch given for each iteration (default: ``32``)
    epochs: int
        number of iterations we train the model (default: ``10``)
    window_size: int
        number of datapoints that the model takes once (default: ``10``)
    lstm_layers: int
        number of layers in lstm (default: ``10``)
    early_stopping_delta: float
        If 1 - (loss / last_loss) is less than `delta` for `patience` epochs, stop (default: ``0.05``)
    early_stopping_patience: int
        If 1 - (loss / last_loss) is less than `delta` for `patience` epochs, stop (default: ``10``)

    Parameters
    ----------
    params : Optional[ParameterConfig]
        Parameter configuration for the algorithm
    skip_pull : bool
        Set to ``True`` to skip pulling the Docker image and use a local image instead.
        If the image is not present locally, this will raise an error.
    timeout : Optional[Duration]
        Set an individual execution and training timeout for this algorithm.
        This will overwrite the global timeouts set using :class:`~timeeval.ResourceConstraints`.

    Returns
    -------
    ~timeeval.Algorithm
        A correctly configured :class:`~timeeval.Algorithm` object for the LSTM-VAE algorithm.
    """
    return Algorithm(
        name="LSTM-VAE",
        main=DockerAdapter(
            image_name="ghcr.io/timeeval/lstm_vae",
            tag="0.3.0",
            skip_pull=skip_pull,
            timeout=timeout,
            group_privileges="akita",
        ),
        preprocess=None,
        postprocess=None,
        param_schema=_lstm_vae_parameters,
        param_config=params or ParameterConfig.defaults(),
        data_as_file=True,
        training_type=TrainingType.SEMI_SUPERVISED,
        input_dimensionality=InputDimensionality("univariate")
    )
