# DO NOT EDIT THIS FILE!
# This file was automatically generated using the timeeval_experiments.generator from the template:
# timeeval_experiments/generator/templates/docker-algorithm.py.jinja
from durations import Duration
from typing import Any, Dict, Optional

from timeeval import Algorithm, TrainingType, InputDimensionality
from timeeval.adapters import DockerAdapter
from timeeval.params import ParameterConfig

import numpy as np


from timeeval.utils.window import ReverseWindowing
# post-processing for valmod
def post_valmod(scores: np.ndarray, args: dict) -> np.ndarray:
    window_min = args.get("hyper_params", {}).get("min_anomaly_window_size", 30)
    window_min = max(window_min, 4)
    return ReverseWindowing(window_size=window_min).fit_transform(scores)


_valmod_parameters: Dict[str, Dict[str, Any]] = {
 "exclusion_zone": {
  "defaultValue": 0.5,
  "description": "Size of the exclusion zone as a factor of the window_size. This prevents self-matches.",
  "name": "exclusion_zone",
  "type": "Float"
 },
 "heap_size": {
  "defaultValue": 50,
  "description": "Size of the distance profile heap buffer",
  "name": "heap_size",
  "type": "Int"
 },
 "max_anomaly_window_size": {
  "defaultValue": 40,
  "description": "Maximum sliding window size",
  "name": "max_anomaly_window_size",
  "type": "Int"
 },
 "min_anomaly_window_size": {
  "defaultValue": 30,
  "description": "Minimum sliding window size",
  "name": "min_anomaly_window_size",
  "type": "Int"
 },
 "random_state": {
  "defaultValue": 42,
  "description": "Seed for random number generation.",
  "name": "random_state",
  "type": "Int"
 },
 "verbose": {
  "defaultValue": 1,
  "description": "Controls logging verbosity.",
  "name": "verbose",
  "type": "Int"
 }
}


def valmod(params: Optional[ParameterConfig] = None, skip_pull: bool = False, timeout: Optional[Duration] = None) -> Algorithm:
    """VALMOD

    Implementation of https://doi.org/10.1007/s10618-020-00685-w.


    **Algorithm Parameters:**

    min_anomaly_window_size: Int
        Minimum sliding window size (default: ``30``)
    max_anomaly_window_size: Int
        Maximum sliding window size (default: ``40``)
    heap_size: Int
        Size of the distance profile heap buffer (default: ``50``)
    exclusion_zone: Float
        Size of the exclusion zone as a factor of the window_size. This prevents self-matches. (default: ``0.5``)
    verbose: Int
        Controls logging verbosity. (default: ``1``)
    random_state: Int
        Seed for random number generation. (default: ``42``)

    Parameters
    ----------
    params : Optional[ParameterConfig]
        Parameter configuration for the algorithm
    skip_pull : bool
        Set to ``True`` to skip pulling the Docker image and use a local image instead.
        If the image is not present locally, this will raise an error.
    timeout : Optional[Duration]
        Set an individual execution and training timeout for this algorithm.
        This will overwrite the global timeouts set using :class:`~timeeval.ResourceConstraints`.

    Returns
    -------
    ~timeeval.Algorithm
        A correctly configured :class:`~timeeval.Algorithm` object for the VALMOD algorithm.
    """
    return Algorithm(
        name="VALMOD",
        main=DockerAdapter(
            image_name="ghcr.io/timeeval/valmod",
            tag="0.3.0",
            skip_pull=skip_pull,
            timeout=timeout,
            group_privileges="akita",
        ),
        preprocess=None,
        postprocess=post_valmod,
        param_schema=_valmod_parameters,
        param_config=params or ParameterConfig.defaults(),
        data_as_file=True,
        training_type=TrainingType.UNSUPERVISED,
        input_dimensionality=InputDimensionality("univariate")
    )
